// file black_scholes_imp_vol_newt.cc
// author: Bernt A Oedegaard
// calculate implied volatility of Black Scholes formula using newton steps

#include "fin_algoritms.h"
#include "normdist.h"
#include <cmath>

double option_price_implied_volatility_call_black_scholes_newton(
     double S, double X, double r, double time, double option_price)
{
  // check for arbitrage violations:
  // if price at almost zero volatility greater than price, return 0
  double sigma_low = 1e-5;
  double price = option_price_call_black_scholes(S,X,r,sigma_low,time);
  if (price > option_price) return 0.0;

  const int MAX_ITERATIONS = 100;
  const double ACCURACY    = 1.0e-4; 
  double t_sqrt = sqrt(time);

  double sigma = (option_price/S)/(0.398*t_sqrt);    // find initial value
  for (int i=0;i<MAX_ITERATIONS;i++){
    price = option_price_call_black_scholes(S,X,r,sigma,time);
    double diff = option_price -price;
    if (fabs(diff)<ACCURACY) return sigma;
    double d1 = (log(S/X)+r*time)/(sigma*t_sqrt) + 0.5*sigma*t_sqrt; 
    double vega = S * t_sqrt * n(d1);
    sigma = sigma + diff/vega;
  };
  return -99e10;  // something screwy happened, should throw exception
};
