// black_scholes_partials_call.cc
// author: Bernt A Oedegaard
// The partial derivatives of the Black - Scholes formula

#include <cmath>
#include "normdist.h"               

void option_price_partials_call_black_scholes( double S,     // spot price
					       double X,     // Strike (exercise) price,
					       double r,     // interest rate
					       double sigma, // volatility
					       double time,  // time to maturity
					       double& Delta, //  partial wrt S
					       double& Gamma, //  second prt wrt S
					       double& Theta, // partial wrt time
					       double& Vega,  //  partial wrt sigma
					       double& Rho){   // partial wrt r
  double time_sqrt = sqrt(time);
  double d1 = (log(S/X)+r*time)/(sigma*time_sqrt) + 0.5*sigma*time_sqrt; 
  double d2 = d1-(sigma*time_sqrt);

  Delta = N(d1);
  Gamma = n(d1)/(S*sigma*time_sqrt);
  Theta =- (S*sigma*n(d1)) / (2*time_sqrt) - r*X*exp( -r*time)*N(d2);
  Vega  = S * time_sqrt * n(d1);
  Rho = X * time * exp(-r * time) * N(d2);
};
