// file: black_scholes_put
// author: Bernt A Oedegaard
//    Calculation of the Black Scholes option price formula.

#include <cmath>              // mathematical library
#include "normdist.h"          // this defines the normal distribution
 

double option_price_put_black_scholes( double S,      // spot price
				       double X,      // Strike (exercise) price,
				       double r,      // interest rate
				       double sigma,  // volatility
				       double time) {  
    double time_sqrt = sqrt(time);
    double d1 = (log(S/X)+r*time)/(sigma*time_sqrt) + 0.5*sigma*time_sqrt;
    double d2 = d1-(sigma*time_sqrt);
    double p =  X * exp(-r*time) * N(-d2) - S * N(-d1); 
    return p;
};
