// file: fin_algoritms.h
// author: Bernt Arne Oedegaard
//   defines all routines in the collection of algoritms.

#include <vector>

///  if you have my term structure class, remove comment in next line 
//  #define _HAVE_TERM_STRUCTURE_CLASS_

/// if you have installed the NEWMAT09 matrix library, uncomment below
// #define _HAVE_NEWMAT_

///////// cash flow ////////////////////////////////////

double cash_flow_pv (vector<double>& cflow_times, 
		     vector<double>& cflow_amounts, double r);

double cash_flow_pv_discrete (vector<double>& cflow_times, 
			      vector<double>& cflow_amounts, double r);

double cash_flow_irr(vector<double>& cflow_times, 
		     vector<double>& cflow_amounts);

bool cash_flow_unique_irr(vector<double>& cflow_times, 
			  vector<double>& cflow_amounts);


//// analytical solutions, option pricing (black scholes etc) /////////////////

double option_price_call_black_scholes( 
   double S, double X, double r, double sigma, double time) ;

double option_price_put_black_scholes( 
   double S, double X, double r, double sigma, double time) ;

double option_price_implied_volatility_call_black_scholes_newton(
  double S, double X, double r, double time, double option_price);

double option_price_implied_volatility_call_black_scholes_bisections(
  double S, double X, double r, double time, double option_price);

double option_price_delta_call_black_scholes(
  double S, double X, double r, double sigma, double time);

double option_price_delta_put_black_scholes(
  double S, double X, double r, double sigma, double time);

void option_price_partials_call_black_scholes(
   double S, double X, double r, double sigma, double time, 
   double& Delta, double& Gamma, double& Theta, double& Vega, double& Rho);

void option_price_partials_put_black_scholes(
   double S, double X, double r, double sigma, double time, 
   double& Delta, double& Gamma, double& Theta, double& Vega, double& Rho);

double option_price_european_call_payout( 
   double S, double X, double r, double b, double sigma, double time);

double option_price_european_put_payout( 
   double S, double X, double r, double b, double sigma, double time);

double option_price_european_call_dividends(
    double S, double X, double r, double sigma, double time_to_maturity,
    vector<double>& dividend_times, vector<double>& dividend_amounts );

double option_price_european_put_dividends(
    double S, double X, double r, double sigma, double time_to_maturity,
    vector<double>& dividend_times, vector<double>& dividend_amounts );

double option_price_american_call_dividend(
   double S, double X, double r, double sigma, double tau,
   double D1, double tau1);

// binomial option pricing ////////////////

double option_price_call_european_binomial(double S,double X,double r,double sigma,double t,int steps);
double option_price_put_european_binomial(double S, double X, double r, double sigma, double t, int steps);

double option_price_call_american_binomial(double S,double X,double r,double sigma,double t,int steps);
double option_price_put_american_binomial(double S, double X, double r, double sigma, double t, int steps);
 
double option_price_call_american_discrete_dividends_binomial(
   double S, double X, double r, double sigma,  double t, int steps, 
   vector<double>& dividend_times, vector<double>& dividend_amounts);

double option_price_put_american_discrete_dividends_binomial(
   double S, double X, double r, double sigma,  double t, int steps, 
   vector<double>& dividend_times, vector<double>& dividend_amounts);

double option_price_call_american_proportional_dividends_binomial(
    double S, 
    double X,
    double r,
    double sigma, 
    double time, 
    int no_steps,
    vector<double>& dividend_times,
    vector<double>& dividend_yields);

double option_price_put_american_proportional_dividends_binomial(
    double S, 
    double X,
    double r,
    double sigma, 
    double time, 
    int no_steps,
    vector<double>& dividend_times,
    vector<double>& dividend_yields); 

double option_price_delta_american_call_binomial(double S,
					 double X,  
					 double r,  
					 double sigma,
					 double t, 
					 int no_steps);

double option_price_delta_american_put_binomial(double S,
					 double X,  
					 double r,  
					 double sigma,
					 double t, 
					 int no_steps);

void option_price_partials_american_call_binomial(double S,
						  double X,
					 double r,    
					 double sigma,
					 double time, 
					 int no_steps,
					 double& delta,
					 double& gamma,
					 double& theta,
					 double& vega, 
					 double& rho);  

 
void option_price_partials_american_put_binomial(double S,
						  double X,
					 double r,    
					 double sigma,
					 double time, 
					 int no_steps,
					 double& delta,
					 double& gamma,
					 double& theta,
					 double& vega, 
					 double& rho);  

//////////////////// finite differences //////////////////

double option_price_call_american_finite_diff_explicit( 
   double S, double X, double  r, double sigma, double time, 
   int no_S_steps, int no_t_steps);

double option_price_put_american_finite_diff_explicit( 
   double S, double X, double  r, double sigma, double time, 
   int no_S_steps, int no_t_steps);

double option_price_call_european_finite_diff_explicit( 
   double S, double X, double  r, double sigma, double time, 
   int no_S_steps, int no_t_steps);

double option_price_put_european_finite_diff_explicit( 
   double S, double X, double  r, double sigma, double time, 
   int no_S_steps, int no_t_steps);

double option_price_call_american_finite_diff_implicit( 
   double S, double X, double  r, double sigma, double time, 
   int no_S_steps, int no_t_steps);

double option_price_put_american_finite_diff_implicit( 
   double S, double X, double  r, double sigma, double time, 
   int no_S_steps, int no_t_steps);

double option_price_call_european_finite_diff_implicit( 
   double S, double X, double  r, double sigma, double time, 
   int no_S_steps, int no_t_steps);

double option_price_put_european_finite_diff_implicit( 
   double S, double X, double  r, double sigma, double time, 
   int no_S_steps, int no_t_steps);

///////////////////////// simulated option prices //////////////////////////////////////

// file simulated.h

double option_price_call_european_simulated(
   double S, double X, double r, double sigma, double time, int no_sims);

double option_price_put_european_simulated(
   double S, double X, double r, double sigma, double time, int no_sims);

double option_price_delta_call_european_simulated(
   double S, double X, double r, double sigma, double time, int no_sims);

double option_price_delta_put_european_simulated(
   double S, double X, double r, double sigma, double time, int no_sims);

/////////// approximated option prices ////////////////////////

double option_price_american_call_approximated_baw(
 double S, double X, double r, double b, double sigma, double time);

double option_price_american_put_approximated_baw(
 double S, double X, double r, double b, double sigma, double time);

//////////////////////////////  futures ///////////////////

double futures_price(double S, double r, double time_to_maturity);

double futures_option_price_call_european_black(
    double F, double X, double r, double sigma, double time);

double futures_option_price_put_european_black(
    double F, double X, double r, double sigma, double time);

double futures_option_price_call_american_binomial(
    double F, double X, double r, double sigma, double time, int no_steps);

double futures_option_price_put_american_binomial(
    double F, double X, double r, double sigma, double time, int no_steps);

//////////////// currency ////////////////////

double currency_option_price_call_european( 
   double S, double X, double r, double r_f, double sigma, double time);

double currency_option_price_put_european(
    double S, double X, double r, double r_f, double sigma, double time);

double currency_option_price_call_american_binomial(
   double S, double X, double r, double r_f, double sigma, double t, int n);

double currency_option_price_put_american_binomial(
   double S, double X, double r, double r_f, double sigma, double t, int n);

////////////// exotic options ////////////////////////////////

double exotics_lookback_european_call(double S,
				      double Smin,
				   double r,
				   double q,
				   double sigma,
				   double time);

double exotics_lookback_european_put(double S,
				   double Smin,
				   double r,
				   double q,
				   double sigma,
				   double time);
///////////////////////// general simulation ////////////////////////

double derivative_price_european_simulated ( 
   double S,
   double X,
   double r,
   double sigma,
   double time,
   double payoff(double& price, double& X),
   int no_sims);

double derivative_price_european_simulated ( 
   double S,
   double r,
   double sigma,
   double time,
   double payoff(vector<double>& prices),
   int no_steps,
   int no_sims);

double derivative_price_european_simulated_control_variate ( 
   double S,
   double r,
   double sigma,
   double time,
   double payoff(vector<double>& prices),
   int no_steps,
   int no_sims); 

double simulate_terminal_price(double S,  // current value of underlying
			       double r,  // interest rate
			       double sigma,  // volatitily 
			       double time);  // time to final date

void simulate_price_sequence(double S,  // current value of underlying
			     double r,  // interest rate
			     double sigma,  // volatitily 
			     double time,  // time to final date
			     int no_steps,  // number of steps 
			     vector<double>& prices );

double payoff_european_call(double& price, double& X);
double payoff_european_put(double& price, double& X);
double payoff_min(vector<double>& prices); 
double payoff_max(vector<double>& prices); 
double payoff_geometric_average(vector<double>& prices);
double payoff_arithmetric_average(vector<double>& prices);

/////////////////// alternative stochastic processes ////////////////

double option_price_call_merton_jump_diffusion( double S, 
						double X,
						double r, 
						double sigma, 
						double time_to_maturity,
						double lambda,
						double kappa,
						double delta);

//////////////////////////////// classical bonds pricing ////////////

double bonds_price(const vector<double>& coupon_times,
		   const vector<double>& coupon_amounts,
		   const vector<double>& principal_times, 
		   const vector<double>& principal_amounts, 
		   const double& r);

double bonds_price(const vector<double>& cashflow_times,
		   const vector<double>& cashflows,
		   const double& r);  

double bonds_price_discrete(const vector<double>& cashflow_times,
			    const vector<double>& cashflows,
			    const double& r);  

double bonds_duration(const vector<double>& cashflow_times,
		      const vector<double>& cashflows,
		      const double& r);

double bonds_yield_to_maturity( const vector<double>& cashflow_times, 
				const vector<double>& cashflow_amounts,
				const double& bondprice);

double bonds_duration_macaulay(const vector<double>& cashflow_times,
			       const vector<double>& cashflows,
			       const double& bond_price);

double bonds_duration_modified (const vector<double>& cashflow_times,
				const vector<double>& cashflow_amounts,
				const double& bond_price,
				const double& r);
 
double bonds_convexity(const vector<double>& cashflow_times,   
		       const vector<double>& cashflow_amounts,
		       const double& y );


#ifdef _HAVE_TERM_STRUCTURE_CLASS_
#include "term_structure_class.h"

double bonds_price(const vector<double>& cashflow_times,  
		   const vector<double>& cashflows,
		   const term_structure_class& d); 

double bonds_duration(const vector<double>& cashflow_times,
		      const vector<double>& cashflow_amounts,
		      const term_structure_class& d); 

double bonds_duration_modified (const vector<double>& cahflow_times,
				const vector<double>& cashflow_amounts,
				const double& bond_price,
				const term_structure_class& d);

#endif

//////// term structure ////////////////////////////////////////////////////

double term_structure_yield_from_discount_factor(double dfact, double t); 
double term_structure_discount_factor_from_yield(double r, double t);
double term_structure_forward_rate_from_disc_facts(double d_t, double d_T,
						   double time);
double term_structure_forward_rate_from_yields(double r_t1, double r_T, 
					       double t1, double T);

double term_structure_discount_factor_cubic_spline(double t,
						   double b1,
						   double c1,
						   double d1,
						   const vector<double>& f,
						   const vector<double>& knots);

double term_structure_discount_factor_vasicek(double time,
					      double r,
					      double a,double b, double sigma);

double term_structure_discount_factor_cir(double t, double r,
					  double kappa,
					  double lambda,
					  double theta,
					  double sigma);

double term_structure_discount_factor_estimated_cir(
    double t, double r, double phi1, double phi2, double phi3);

double term_structure_yield_linearly_interpolated(double time, 
						  const vector<double>& obs_times, 
						  const vector<double>& obs_yields);

double term_structure_yield_nelson_siegel(double t,
					  double beta0, double beta1, double beta2,
					  double lambda );

double term_structure_yield_bliss(double t,
				  double gamma0, double gamma1, double gamma2,
				  double lambda1, double lambda2);

///////////////////////////////////////////////////////////////////////////
// fixed income derivatives, 
// Black Scholes bond option

double bond_option_price_call_zero_black_scholes(
    double B, double X, double r, double sigma, double time);

double bond_option_price_put_zero_black_scholes(
    double B, double X, double r, double sigma, double time);

double bond_option_price_call_coupon_bond_black_scholes(
    double B, double X, double r, double sigma, double time, 
    vector<double> coupon_times, vector<double> coupon_amounts);

double bond_option_price_put_coupon_bond_black_scholes(
    double B, double X, double r, double sigma, double time, 
    vector<double> coupon_times, vector<double> coupon_amounts);

/// rendlemann barterr 

double bond_option_price_call_zero_american_rendleman_bartter(
   double X,  
   double option_maturity, 
   double S, 
   double M,                   // term structure paramters
   double interest,            // current short interest rate
   double bond_maturity,        // time to maturity for underlying bond
   double maturity_payment,
   int no_steps);

/// Vasicek model, explicit call 

double bond_option_price_call_zero_vasicek(double X, // exercise price
					   double r, // current interest rate
					   double option_time_to_maturity,
					   double bond_time_to_maturity,
					   double a,  // parameters
					   double b,  
					   double sigma);

double bond_option_price_put_zero_vasicek(double X,  
					  double r, 
					   double option_time_to_maturity,
					   double bond_time_to_maturity,
					   double a, 
					   double b, 
					   double sigma);

