// file: findiff_exp_am_call.cc
// author: Bernt A Oedegaard

#include <cmath>
#include <algorithm>
#include <vector>

double option_price_call_american_finite_diff_explicit(double S,  
						       double X, 
						       double r,
						       double sigma,
						       double time,
						       int no_S_steps,
						       int no_t_steps) {
   double sigma_sqr = sigma*sigma;
   int M=no_S_steps;           // need M = no_S_steps to be even:
   if ((no_S_steps%2)==1) { M=no_S_steps+1; } else { M=no_S_steps; };
   double delta_S = 2.0*S/M;
   vector<double> S_values(M+1,0.0);
   for (int m=0;m<=M;m++) { S_values[m] = m*delta_S; };
   int N=no_t_steps;
   double delta_t = time/N;
    
   vector<double> a(M,0.0);
   vector<double> b(M,0.0);
   vector<double> c(M,0.0);
   double r1=1.0/(1.0+r*delta_t);
   double r2=delta_t/(1.0+r*delta_t);
   for (int j=1;j<M;j++){
      a[j] = r2*0.5*j*(-r+sigma_sqr*j);
      b[j] = r1*(1.0-sigma_sqr*j*j*delta_t);
      c[j] = r2*0.5*j*(r+sigma_sqr*j);
   };
   vector<double> f_next(M+1,0.0);
   for (int m=0;m<=M;++m) { f_next[m]=max(0.0,S_values[m]-X); };
   vector<double> f(M+1,0.0);
   for (int t=N-1;t>=0;--t) {
      f[0]=0;
      for (int m=1;m<M;++m) {
	 f[m]=a[m]*f_next[m-1]+b[m]*f_next[m]+c[m]*f_next[m+1];
	 f[m] = max(f[m],S_values[m]-X);  // check for exercise
      };
      f[M] = S_values[M]-X;
      for (int m=0;m<=M;++m) { f_next[m] = f[m]; };
   };
   double C = f[M/2];
   return C;
};
