// findiff_imp_am_call.cc
// author: Bernt A Oedegaard

#include <cmath>
#include "newmat.h"  // definitions for newmat matrix library
#include <vector>   // standard STL vector template
#include <algorithm>

double option_price_call_american_finite_diff_implicit(double S, double X,
						       double  r, double sigma, double time, 
						       int no_S_steps, int no_t_steps) {
    double sigma_sqr = sigma*sigma;

    int M; if ((no_S_steps%2)==1) { M=no_S_steps+1; } else { M=no_S_steps; }; // need no_S_steps to be even:
    double delta_S = 2.0*S/M;
    vector<double> S_values(M+1);
    for (unsigned m=0;m<=M;m++) { S_values[m] = m*delta_S; };
    int N=no_t_steps;
    double delta_t = time/N;
    
    BandMatrix A(M+1,1,1); A=0.0;
    A.element(0,0) = 1.0;
    for (int j=1;j<M;++j) {
	A.element(j,j-1) = 0.5*j*delta_t*(r-sigma_sqr*j);    // a[j]
	A.element(j,j)   = 1.0 + delta_t*(r+sigma_sqr*j*j);  // b[j];
	A.element(j,j+1) = 0.5*j*delta_t*(-r-sigma_sqr*j);   // c[j];
    };
    A.element(M,M)=1.0;
    ColumnVector B(M+1);
    for (unsigned m=0;m<=M;++m){ B.element(m) = max(0.0,S_values[m]-X); };
    ColumnVector F=A.i()*B;
    for(int t=N-1;t>0;--t) {
	B = F;
	F = A.i()*B;
	for (unsigned m=1;m<M;++m) { 	// now check for exercise
	    F.element(m) = max(F.element(m), S_values[m]-X);
	};
    };
    return F.element(M/2);
};
