// file futures_opt_call_black.cc
// author: Bernt A Oedegaard

#include <cmath>                                 // mathematics library
#include "normdist.h"                             // normal distribution

double futures_option_price_call_european_black(
   double F,     // futures price
   double X,     // exercise price
   double r,     // interest rate
   double sigma, // volatility
   double time)   // time to maturity
{
    double sigma_sqr = sigma*sigma;
    double time_sqrt = sqrt(time);
    double d1 = (log (F/X) + 0.5 * sigma_sqr * time) / (sigma * time_sqrt);
    double d2 = d1 - sigma * time_sqrt;
    return exp(-r*time)*(F * N(d1) - X * N(d2));
};


