// file futures_opt_call_bin.cc
// author: Bernt A Oedegaard

#include <cmath>
#include <algorithm>
#include <vector>

double futures_option_price_put_american_binomial(
    double F, double X, double r, double sigma, double time, int no_steps){
   vector<double> futures_prices(no_steps+1);
   vector<double> put_values (no_steps+1);
   double t_delta= time/no_steps;
   double Rinv = exp(-r*(t_delta));
   double u = exp(sigma*sqrt(t_delta));
   double d = 1.0/u;
   double uu= u*u;
   double uInv=1.0/u;
   double pUp   = (1-d)/(u-d);
   double pDown = 1.0 - pUp;
   futures_prices[0] = F*pow(d, no_steps);
   int i;
   for (i=1; i<=no_steps; ++i) 
      futures_prices[i] = uu*futures_prices[i-1]; // terminal tree nodes
   for (i=0; i<=no_steps; ++i) put_values[i] = max(0.0, (X-futures_prices[i]));
   for (int step=no_steps-1; step>=0; --step) {
      for (i=0; i<=step; ++i)   {
	 futures_prices[i] = uInv*futures_prices[i+1];
	 put_values[i] = (pDown*put_values[i]+pUp*put_values[i+1])*Rinv;
	 put_values[i] = max(put_values[i], X-futures_prices[i]); // check for exercise
      };
   };
   return put_values[0];
};
