// file futures_opt_put_black.cc
// author: Bernt A Oedegaard

#include <cmath>                                 // mathematics library
#include "normdist.h"                             // normal distribution

double futures_option_price_put_european_black ( double F,     // futures price
						double X,     // exercise price
						double r,     // interest rate
						double sigma, // volatility
						double time) 
{
    double sigma_sqr = sigma*sigma;
    double time_sqrt = sqrt(time);
    double d1 = (log (F/X) + 0.5 * sigma_sqr * time) / (sigma * time_sqrt);
    double d2 = d1 - sigma * time_sqrt;
    return  exp(-r*time)*(X * N(-d2) - F * N(-d1));
};
