// file merton_jump_diff_call.cc 
// author: Bernt Arne Oedegaard

#include <cmath>
#include "fin_algoritms.h"

double option_price_call_merton_jump_diffusion( double S, 
						double X,
						double r, 
						double sigma, 
						double time_to_maturity,
						double lambda,
						double kappa,
						double delta) {
    const int MAXN=50;
    double tau=time_to_maturity;
    double sigma_sqr = sigma*sigma; 
    double delta_sqr = delta*delta;
    double lambdaprime = lambda * (1+kappa);
    double gamma = log(1+kappa);
    double c = exp(-lambdaprime*tau)*
	option_price_call_black_scholes(S,X,sigma,r-lambda*kappa,tau);
    double log_n = 0;
    for (int n=1;n<=MAXN; ++n) {
	log_n += log(n);
	double sigma_n = sqrt( sigma_sqr+n*delta_sqr/tau );
	double r_n = r-lambda*kappa+n*gamma/tau;
	c += exp(-lambdaprime*tau+n*log(lambdaprime*tau)-log_n)*
	    option_price_call_black_scholes(S,X,sigma_n,r_n,tau);
    };
    return c;
};
