/* file random.cc
   author Bernt A Oedegaard

   Interface to PD random number generators. 

   This interface meant for program that are not sophisticated about which
   generators are used, seeds etc.  Used if the only purpose is to get
   reasonable random numbers.

*/

#include "knuthran.h"  // Knuth code in C.
#include <time.h>
#include <math.h>

double random_normal(void){
    // normal number generation mean 0, variance 1
    // note: seed is used and changed internally.
   static unsigned int seed = clock()%100000;  // use system clock if want truly random numbers
//   static unsigned int seed = 1;     // pick your own seed if you want replicability
    return p_dNormal(0.0,1.0,&seed);
};

double random_normal(const double mu, const double sigma){
    // normal number generation given mean , variance 
    static unsigned int seed = 1;
    return p_dNormal(mu,sigma,&seed);
};

double random_log_normal(const double mu, const double sigma){
    // normal number generation given mean , variance 
  return exp(random_normal(mu,sigma));
};

double random_uniform(double lo, double hi){
    // uniform [low, high] random number
    static unsigned int seed=1;
    double u = p_dUniform(&seed);
    double r = lo + (hi-lo) * u ;
    return r;
};

double random_uniform(void){
    // uniform [0,1]
    static unsigned int seed=1;
    return p_dUniform(&seed);
};

