// file simulated_delta_put.cc
// author: Bernt Arne Oedegaard 

#include <cmath>
#include <algorithm>
#include "random.h"

double option_price_delta_put_european_simulated(double S, 
						 double X, 
					   double r,
					   double sigma, 
					   double time,
					   int no_sims)
   // estimate the price using two different S values
{
    double sigma_sqr = sigma * sigma;
    double R = (r - 0.5 * sigma_sqr)*time;
    double SD = sigma * sqrt(time);
    double sum_payoffs = 0.0;
    double sum_payoffs_2 = 0.0;
    double q = S*0.01; 
    for (int n=1; n<=no_sims; n++) {
      double Z = random_normal();
      double S_T = S * exp(R + SD * Z);
      double S_T_2 = (S+q) * exp(R + SD * Z);
      sum_payoffs += max(0.0, X-S_T);
      sum_payoffs_2 += max(0.0, X-S_T_2);
    };
    double p = exp(-r*time) * ( sum_payoffs/no_sims); 
    double p2 = exp(-r*time) * ( sum_payoffs_2/no_sims); 
    return (p2-p)/q;
};
